#!/usr/bin/env python
# encoding: utf-8
"""
gui.py

This is the graphical section for the client.

Created by Elliot Pennington on 2008-12-15.
"""


from Queue import *
from Tkinter import *
import tkMessageBox
import threading
import receiver

class Window:
    
    """
    Tabbed window hackup
    
    Let's see if this works
    """
    
    def __init__(self, master):
        self.active_fr = None
        self.count = 0
        self.choice = IntVar(0)
        self.side = LEFT
        
        self.rb_fr = Frame(master, borderwidth = 2, relief = RIDGE)
        self.rb_fr.pack(fill = BOTH)
        
        self.screen_fr = Frame(master, borderwidth = 2, relief = RIDGE)
        self.screen_fr.pack(fill = BOTH)
        
    def __call__(self):
        return self.screen_fr
    
    def add_screen(self, fr, title):
        b = Radiobutton(self.rb_fr, text = title, indicatoron = 0, \
                        variable = self.choice, value = self.count, \
                        command = lambda: self.display(fr))
        b.pack(fill = BOTH, side = self.side)
        
        if not self.active_fr:
            fr.pack(fill = BOTH, expand = 1)
            self.active_fr = fr
        
        self.count += 1
        
        return b

    def display(self, fr):
        self.active_fr.forget()
        fr.pack(fill = BOTH, expand = 1)
        self.active_fr = fr


class TabbedFrame (Frame):
    
    """
    Special frame for tabbing in the Window class
    """
    
    def __init__(self, master, sn, in_queue, out_queue, in_event, out_event):
        Frame.__init__(self, master)
        
        self.sn = sn
        self.in_queue = in_queue
        self.out_queue = out_queue 
        self.out_event = out_event
        self.in_event = in_event
        
        self.topframe = Frame(self)
        self.midframe = Frame(self)
        
        # Configure TOPFRAME
        Label (self, text='Conversation with: %s' % self.sn).pack(side=TOP,padx=10,pady=10)
    
        scroll = Scrollbar(self)
        scroll2 = Scrollbar(self)
        
        self.msg_txt = Text(self.topframe, padx = 10, pady = 10, width = 65, height = 20)
        self.msg_txt.configure(yscrollcommand = scroll.set)
        self.msg_txt.pack(side = LEFT)
        scroll.config(command=self.msg_txt.yview)
        scroll.pack(side = RIGHT, fill=Y)
        self.topframe.pack(side = TOP)
        self.msg_txt.configure(state=DISABLED)
        
        # Configure MIDFRAME
        self.entry_frame = Text(self.midframe, width = 60, height = 3)
        self.entry_frame.configure(yscrollcommand = scroll2.set)
        self.entry_frame.pack(side=TOP, padx = 10, pady = 10)
        scroll2.config(command=self.entry_frame.yview)
        scroll2.pack(side=RIGHT, fill=Y)
        self.midframe.pack(side = BOTTOM)
        self.entry_frame.focus_set()
        
        #Buttons        
        self.send = Button(self, text="Send", width=10, command=self.out)
        self.send.pack(side = RIGHT)
        
        self.end = Button(self, text="End Conversation", width=15, command=self.end_Conversation)
        self.end.pack(side = LEFT)
        
    def out(self):
        data = self.entry_frame.get("0.0", END)
        self.entry_frame.delete("0.0", END)
        
        self.print_queue.put("%s" % data, True)
        self.print_event.set()
        self.print_event.clear()
        
        self.out_queue.put("%s::en:%s" % (self.sn, data), True)
        self.out_event.set()
        self.out_event.clear()
    
    def end_Conversation(self):
        if tkMessageBox.askokcancel("End Conversation", "Are you sure you want to end the conversation?"):
            self.out_queue.put("~server:endConvo:%s" % self.sn, True) 
            self.out_event.set()
            self.out_event.clear()
# TODO: Open notes window, remove tab, remove from client conversation list
            self.destroy()
      		
    def spawn_msg_frame(self, sn, q, ev):
        t = threading.Thread(target=self.spawn_receiver,
                             args=(sn, q, ev, self.msg_txt.insert, self.msg_txt.configure))
        t.start()


    def spawn_receiver(self, sn, in_queue, in_event, insert, config):
        receiver.Receiver(sn, in_queue, in_event, insert, config)
        
"""    
class c_window:
    
    def __init__(self, sn, in_queue, out_queue, in_event, out_event, top_event):
#       Set up window for conversation

        self.sn = sn
        self.in_queue = in_queue
        self.out_queue = out_queue 
        self.out_event = out_event
        self.in_event = in_event
        
        self.print_queue = Queue() 
        self.print_event = threading.Event()
        
        self.root2 = Tk()
        self.root2.withdraw()
        self.root = Toplevel(self.root2)
        self.root.protocol("WM_DELETE_WINDOW", self.end_Conversation)
        self.root.title("Conversation with %s" % self.sn)

        self.frame = Frame(self.root)
        
        #frames to hopefully fix scrollbar packing  
        self.topframe = Frame(self.frame)
        self.midframe = Frame(self.frame)
        
        Label (self.frame, text='Conversation with: %s' % self.sn).pack(side=TOP,padx=10,pady=10)
    
        scroll = Scrollbar(self.frame)
        scroll2 = Scrollbar(self.frame)
#topframe
        self.msg_txt = Text(self.topframe, padx = 10, pady = 10, width = 65, height = 20)
        self.msg_txt.configure(yscrollcommand = scroll.set)
        self.msg_txt.pack(side = LEFT)
        scroll.config(command=self.msg_txt.yview)
        scroll.pack(side = RIGHT, fill=Y)
        self.topframe.pack(side = TOP)
        self.msg_txt.configure(state=DISABLED)
        
        self.spawn_msg_frame(self.sn, self.in_queue, self.in_event)
        self.spawn_msg_frame("samariteen", self.print_queue, self.print_event)
#midframe        
        self.entry_frame = Text(self.midframe, width = 60, height = 3)
        self.entry_frame.configure(yscrollcommand = scroll2.set)
        self.entry_frame.pack(side=TOP, padx = 10, pady = 10)
        scroll2.config(command=self.entry_frame.yview)
        scroll2.pack(side=RIGHT, fill=Y)
        self.midframe.pack(side = BOTTOM)
        self.entry_frame.focus_set()
#Buttons        
        self.send = Button(self.frame, text="Send", width=10, command=self.out)
        self.send.pack(side = RIGHT)
        
        self.end = Button(self.frame, text="End Conversation", width=15, command=self.end_Conversation)
        self.end.pack(side = LEFT)
        
        self.frame.pack()
        print ("New GUI for conversation with %s" %sn)
        top_event.set()
        top_event.clear()
        top_event.wait()
        self.root.mainloop()
        
    def out(self):
        data = self.entry_frame.get("0.0", END)
        self.entry_frame.delete("0.0", END)
        
        self.print_queue.put("%s" % data, True)
        self.print_event.set()
        self.print_event.clear()
        
        self.out_queue.put("%s::en:%s" % (self.sn, data), True)
        self.out_event.set()
        self.out_event.clear()
    
    def end_Conversation(self):
        if tkMessageBox.askokcancel("End Conversation", "Are you sure you want to end the conversation?"):
            self.out_queue.put("~server:endConvo:%s" % self.sn, True) 
            self.out_event.set()
            self.out_event.clear()
# TODO: Open notes window, remove from current conversation list
            self.root2.destroy()
      
    def spawn_msg_frame(self, sn, q, ev):
        t = threading.Thread(target=self.spawn_receiver,
                             args=(sn, q, ev, self.msg_txt.insert, self.msg_txt.configure))
        t.start()


    def spawn_receiver(self, sn, in_queue, in_event, insert, config):
        receiver.Receiver(sn, in_queue, in_event, insert, config)
"""
class login_window:
    
    """
    We make one of these when we log into the system. It collects the login
    user name and password and sends it to the server for authentication. 
    
    THIS ISN'T FINISHED
    """
    
    def __init__(self, out_queue, out_event, set_username, set_logout):
        
        print "Login window"
        
        self.root2=Tk()
        self.root2.withdraw()
        
        self.root = Toplevel(self.root2)
        self.root.protocol("WM_DELETE_WINDOW", self.close)
        self.root.title("Befriender Login")
        
        self.frame = Frame(self.root)
        self.out_queue = out_queue
        self.out_event = out_event
        self.set_username = set_username
        
        Label (self.frame, text='Login').pack(side=TOP,padx=15,pady=10)
        Label (self.frame, text='Enter your Username & Password').pack(side=TOP)
        
        self.usr_entry = Entry(self.frame, width = 20)
        self.usr_entry.pack(side=TOP, padx = 10, pady = 3)
        self.usr_entry.focus_set()
        
        self.pwd_entry = Entry(self.frame, width = 20, show="*")
        self.pwd_entry.pack(side=TOP,padx = 10, pady = 3)

        self.login = Button(self.frame, text="Login", width=10, command=self.out)
        self.login.pack(side = RIGHT, pady=5)
        self.frame.pack()
        self.root2.mainloop()
    
    def out(self):
        username = self.usr_entry.get()
        password = self.pwd_entry.get()
#        print username
#        print password
        self.out_queue.put("~server:login:%s:%s" % (username, password), True)
        self.out_event.set()
        self.out_event.clear()
        self.set_username(username)
        self.root2.destroy()
        
    def close(self, set_logout):
        if tkMessageBox.askokcancel("Quit", "Are you sure you want to log out?"):
            set_logout()
        

class menu_window(Frame):
    """
    This is the window that appears once login is successful.
    Contains buttons for IM answering availability, logging out, and opening the options window.
    
    THIS ISN'T QUITE FINISHED
    """
    def __init__(self, master, username, out_queue, out_event, set_logout):
        Frame.__init__(self, master)
        self.frame = Frame(self)
        
        self.username = username
        self.out_queue = out_queue
        self.out_event = out_event
        self.set_logout = set_logout
        self.availability = 0
        
        Label (self, text= "Logged in as %s" % username).pack(side=TOP, padx=10, pady=10)
        
        self.avail_label = Label(self.frame, text="Unavailable for conversations", fg="red")
        self.avail_label.pack(side=TOP)     
        
        self.availability_button = Button(self.frame, text="Toggle AIM Availability", width=20, command=self.change_availability)
        self.availability_button.pack(side = TOP, padx=10, pady=10)
        
        self.logout_button = Button(self.frame, text="Logout", width=20, command=self.logout)
        self.logout_button.pack(side = TOP, padx=10, pady=10)
        
        self.frame.pack()
        
    def change_availability(self):
        print "Menu: Availability toggled"
        if(self.availability == 1):
            self.out_queue.put("~server:available:false", True)
            self.out_event.set()
            self.out_event.clear()
            self.avail_label.configure(text = "Unavailable for conversations", fg="red") 
            self.availability = 0
        else:
            self.out_queue.put("~server:available:true", True)
            self.out_event.set()
            self.out_event.clear()
            self.avail_label.configure(text= "Available for conversations", fg="darkgreen")
            self.availability = 1

    
    def logout(self):
        if tkMessageBox.askokcancel("Quit", "Are you sure you want to log out?"):
            self.out_queue.put("~server:logout", True)
            self.out_event.set()
            self.out_event.clear()
            self.set_logout()
            self.destroy()

#TODO:
class options_window(Frame):
    """
    This is the window that appears once the options button is pressed on the menu. 
    If the user has normal permissions has buttons for: Changing password
    If the user has admin permissions has buttons for: Welcome/busy/end conversation messages, 
       changing the notes form, adding a user, changing a users password, and searching 
       the history database
    """
    def __init__(self, master, username, out_queue, out_event):
        Frame.__init__(self, master)
        self.frame = Frame(self)
        self.out_queue = out_queue 
        self.out_event = out_event
        
        Label (self.frame ,text= "Options for %s" % username).pack(side=TOP, padx=10, pady=10)
        self.availability_button = Button(self.frame, text="Option!", width=20, command=self.button)
        self.availability_button.pack(side = TOP, padx=10, pady=10)
        
#TODO(esp): How would I go about getting a users permission level back from the server?        
        self.frame.pack()
    def button(self):
        print "Options button pushed!"
"""        
a = Tk()
n = Window(a)

# uses the notebook's frame
f1 = Frame(n())
b1 = Button(f1, text="Button 1")
e1 = Entry(f1)
# pack your widgets before adding the frame 
# to the notebook (but not the frame itself)!
b1.pack(fill=BOTH, expand=1)
e1.pack(fill=BOTH, expand=1)

f2 = Frame(n())
# this button destroys the 1st screen radiobutton
b2 = Button(f2, text='Button 2', command=lambda:x1.destroy())
b3 = Button(f2, text='Beep 2', command=lambda:n.add_screen(f2, "lol"))
b2.pack(fill=BOTH, expand=1)
b3.pack(fill=BOTH, expand=1)

f3 = Frame(n())

# keeps the reference to the radiobutton (optional)
x1 = n.add_screen(f1, "Screen 1")
n.add_screen(f2, "Screen 2")
n.add_screen(f3, "dummy")

if __name__ == "__main__":
    a.mainloop()

"""


