#!/usr/bin/python

#  msg.py
#  Message class. methods specific to messages are here
#  I suspect that this class is really only relevant for
#  incoming messages, not outgoing messages.
#  
#  Elliot Pennington 2008

import re, toc, unittest

def strip_html(data):
    return re.sub("<[^>]*>","",data)

def normalize(data):
    return re.sub("[^A-Za-z0-9]","",data).lower()

def deserialize(ser):
    """
    Deserialize an array received from socket into a Message object
    Returns an instance of Message
    """
    pat = re.compile("([\w~]*?):(.*)", re.M|re.S)
    gold = pat.match(ser)
    data = gold.group(1) + "::en:" + gold.group(2)
    msg = Message(data)
    return msg

class Message:

    def __init__(self, data):
        pat = re.compile("(\w*?):.*?:en:(.*)", re.M|re.S)
        paydirt = pat.match(data)
        if not paydirt:
            print "found a command, probably"
            paydirt = data.split(':')
            if paydirt:
                self._sn = paydirt[0]
                self._msg = paydirt[1:]
            else:
                print "we've got a big problem here.."
        else: 
            self._sn = paydirt.group(1)
            self._htmlmsg = paydirt.group(2)
            self._sn = normalize(self._sn)
            self._msg = strip_html(self._htmlmsg)
    
    def formatDisplay(self):
        """Format the message in semi-readable format"""
        return "%s: %s" % (self._sn, self._msg)
    
    def display(self):
        """print display to stdout"""
        print self.formatDisplay()

    def encrypt(self):
        import crypt
        """encrypt the screenname with DES"""
        self._sn = crypt.crypt(self._sn, self._sn[0:2])
        
    def routeTo(self, befriender):
        print "routing to " + befriender.login
        
    def serialize(self):
        """
        Serialize a Message instance into an array of bytes.
        Return the resulting array.
        """
        ser = "";
        ser += (self._sn + '::en:' + self._msg)
        return ser;
    
    def getHtmlMsg(self):
        return self._htmlmsg

    def getSn(self):
        return self._sn

    def getMsg(self):
	"""Get the message body without any html"""
        return self._msg

class testMessage(unittest.TestCase):
    
    def setUp(self):
        self.data = "esp1337:F:F:T: UU,:F:A:en:<HTML>omg <I>LOL</I></HTML>"
        self.msg = Message(self.data)
    
    def testSnGroup(self):
        self.assertEqual(self.msg.getSn(), "esp1337")
    
    def testMsgGroup(self):
        self.assertEqual(self.msg._msg, "omg LOL")
    
    def testFormatDisplay(self):
        self.assertEqual(self.msg.formatDisplay(), "esp1337: omg LOL")
        
    def testEncrypt(self):
        self.msg.encrypt()
        self.assertEqual(self.msg._sn, "esEv/CJRxycSE")
        
    def testGetMsg(self):
        self.assertEqual("omg LOL", self.msg.getMsg())
        
    def testSerialize(self):
        self.assertEqual("esp1337: omg LOL", self.msg.serialize())
        
    def testDeserialize(self):
        ser = self.msg.serialize()
        self.assertEqual((self.msg.display()), (deserialize(ser).display()))
    
    
if __name__ == '__main__':
	unittest.main()
